const electron = window.require('electron');
const ipcRenderer = electron.ipcRenderer;

import React, {useState} from 'react';
import {Context} from '../context-provider';

function WebView(props){

    const { appState, appDispatch, browserState, browserDispatch, ocsApiState } = React.useContext(Context);
    const [ webviewEl, setWebviewEl ] = useState();
    const [ showWebviewMask, setShowWebviewMask ] = useState(true);

    React.useEffect(() => { initWebView(browserState.url); },[])

    React.useEffect(() => {
        if (typeof webviewEl !== undefined) props.onSetWebviewEl(webviewEl);
    },[webviewEl])

    React.useEffect(() => {
        if (browserState.goBack === true){
            browserDispatch({type:'GO_BACK',goBack:false})
            webviewEl.goBack();
            setShowWebviewMask(true);
        }
    },[browserState.goBack])

    React.useEffect(() => {
        if (browserState.goForward === true){            
            browserDispatch({type:'GO_FORWARD',goForward:false})
            webviewEl.goForward();
        }
    },[browserState.goForward])

    React.useEffect(() => {
        if (browserState.goHome === true){
            browserDispatch({type:'GO_HOME', goHome:false})
            webviewEl.loadURL('https://www.pling.com');
        }
    },[browserState.goHome])

    React.useEffect(() => {
        if (browserState.refresh === true){
            browserDispatch({type:'REFRESH', refresh:false})
            webviewEl.loadURL(browserState.url);        
        }
    },[browserState.refresh])

    React.useEffect(() => {
        if (browserState.loadUrl === true){
            browserDispatch({type:'LOAD_URL', loadUrl:false})
            webviewEl.loadURL(browserState.newUrl);
        }
    },[browserState.loadUrl])

    React.useEffect(() => {
        if (browserState.stopLoading === true){
            browserDispatch({type:'STOP_LOADING',stopLoading:false});
            webviewEl.stop();
        }
    },[browserState.stopLoading])

    function initWebView(url){

        const webview = document.createElement('webview');
        webview.setAttribute('src', url);
        webview.setAttribute('id',"webview-frame");

        webview.addEventListener('did-start-loading', () => {
            browserDispatch({type:'SET_LOADING',loading:true})
        });

        webview.addEventListener('did-stop-loading', () => {
            browserDispatch({type:'SET_LOADING',loading:false});
            if (appState.dialog.show === true) appDispatch({type:'HIDE_DIALOG'});

            const webViewInitiScript = 
            `var metaheader = document.querySelector('#metaheader'); 
            if (metaheader){ metaheader.parentNode.removeChild(metaheader); }
            var odMetaHeader = document.querySelector('opendesktop-metaheader'); 
            if (odMetaHeader){ odMetaHeader.parentNode.removeChild(odMetaHeader);} 
            var gitLabNavBar = document.querySelector('header.navbar-gitlab');
            if (gitLabNavBar){ gitLabNavBar.style.top = 0;}
            var forumsNavBar = document.querySelector('header.d-header');
            if (forumsNavBar){ forumsNavBar.style.top = 0;}
            var body = document.querySelector('body');
            body.style.paddingTop = 0;`;

            webview.getWebContents().executeJavaScript( webViewInitiScript , false, (result) => {
                setShowWebviewMask(false);
            });
        });

        webview.addEventListener('dom-ready', (event) => {
            browserDispatch({
                type:'SET_PAGE',
                url:webview.getURL(),
                title:webview.getTitle(),
                canGoBack:webview.canGoBack(),
                canGoForward:webview.canGoForward()
            });
            webview.send('ipc-message');
        });

        webview.addEventListener('new-window', (event) => {
            console.log('new window');
            /*if (event.url.startsWith('http://') || event.url.startsWith('https://')) {
                this.dispatch('ocsManager_openUrl', {url: event.url});
            }*/
        });

        webview.addEventListener('will-navigate', (event) => {
            console.log('webview element: will navigate');
            // See also "will-navigate" event handling in main.js
            if (event.url.startsWith('ocs://') || event.url.startsWith('ocss://')) {
                console.log('if event.url starts with ocs://')    
                const info = detectOcsApiInfo(webview.getURL());
                console.log('info:');
                console.log(info);
                console.log('ocs manager - send: item handler::getItemByOcsUrl');
                ocsApiState.ocsManagerApi.send('ItemHandler::getItemByOcsUrl', [event.url, info.providerKey, info.contentId]);
            }
        });

        webview.addEventListener('ipc-message', (event) => {
            console.log('ipc message');
        });

        document.querySelector('#webview-container').appendChild(webview);
        setWebviewEl(webview);
    }

    function detectOcsApiInfo(url) {
        // Detect provider key and content id from page url
        // https://www.opendesktop.org/s/Gnome/p/123456789/?key=val#hash
        //
        // providerKey = https://www.opendesktop.org/ocs/v1/
        // contentId = 123456789
        const info = {
            providerKey: '',
            contentId: ''
        };
        const matches = url.match(/(https?:\/\/[^/]+).*\/p\/([^/?#]+)/);
        if (matches) {
            info.providerKey = `${matches[1]}/ocs/v1/`;
            info.contentId = matches[2];
        }
        return info;
    }

    let webviewMaskDisplay;
    if (showWebviewMask === true){
        webviewMaskDisplay = <div id="webview-mask"></div>
    }

    return (
        <div id="webview-wrapper">
            <div id="webview-container"></div>
            {webviewMaskDisplay}
        </div>
    )
}
  
  export default WebView;